
# Eden's TCL Extensions To Eggdrop  (ETETE)
# by Wayne Brookes, February 1995
# version 0.1


# DEFENCE COMMANDS
#
# This file adds the following 'msg' commands:
#   kick <nick> [reason]
#   kickban <nick> [reason]
#   ban <nick> [reason]
#    or:  ban <nick!user@host> [reason]
#   unban <nick!user@host>
#   giveop <nick> [nick ...]
#   deop   <nick> [nick ...]



#   kick <nick> [reason]

bind msg o kick msg_kick
proc msg_kick {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick kick <nick> \[reason\]"
    return 0
  }
  set who [lindex $arg 0]
  set reason [lrange $arg 1 end]
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }
  if {![botisop]} {
    putserv "NOTICE $nick :I'm not an op on the channel!  I can't help you."
    return 0
  }
  if {![onchan $who]} {
    putserv "NOTICE $nick :$who is not on the channel!"
    return 0
  }
  if {[onchanSplit $who]} {
    putserv "NOTICE $nick :$who is currently net-split."
    return 0
  }
  append userhost $who "!" [getchanhost $who]
  set target [finduser $userhost]
  if {$target != "*"} {
    if {([matchattr $target f] || [matchattr $target m])} {
      putserv "NOTICE $nick :Can't kick a friend or master, sorry."
      return 0
    }
  }
  # check target nickname against bot's current nickname
  if {[string compare [string tolower $who] [string tolower $botnick]] == 0} {
    putserv "NOTICE $nick :Yeah right, like I'm going to let you kick ME!"
    return 0
  }
  # if they gave no reason, use "requested"
  if {$reason == ""} { set reason "requested" }
  putserv "NOTICE $nick :Kicking $who off $channel ($reason)"
  putserv "KICK $channel $who :$reason"
  return 1
}


#   kickban <nick> [reason]

bind msg o kickban msg_kickban
proc msg_kickban {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick kickban <nick> \[reason\]"
    return 0
  }
  set who [lindex $arg 0]
  set reason [lrange $arg 1 end]
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }
  if {![botisop]} {
    putserv "NOTICE $nick :I'm not an op on the channel!  I can't help you."
    return 0
  }
  if {![onchan $who]} {
    putserv "NOTICE $nick :$who is not on the channel!"
    return 0
  }
  append userhost $who "!" [getchanhost $who]
  set target [finduser $userhost]
  if {$target != "*"} {
    if {([matchattr $target f] || [matchattr $target m])} {
      putserv "NOTICE $nick :Can't kickban a friend or master, sorry."
      return 0
    }
  }
  if {[string compare [string tolower $who] [string tolower $botnick]] == 0} {
    putserv "NOTICE $nick :Yeah right, like I'm going to let you kickban ME!"
    return 0
  }
  # if they gave no reason, use "requested"
  if {$reason == ""} { set reason "requested" }
  append bantemp $who "!*" [getchanhost $who]
  set banmask [maskhost $bantemp]
  # If the ban is already on the channel, don't re-ban it, just kick!
  if {[ischanban $banmask]} {
    putserv "NOTICE $nick :That ban is already on the channel.  Kicking $who from $channel ($reason)"
    putserv "KICK $channel $nick :$reason"
    return 1
  }
  # If the ban is already on the bot, don't re-ban it
  if {[isban $banmask]} {
    putserv "NOTICE $nick :That ban is already on the bot.  It will be enforced if necessary.  Kicking $who from $channel ($reason)"
    putserv "KICK $channel $nick :$reason"
    return 1
  }

  # Finally, tidy up the comment to include the banner's handle.
  # (handle cannot be null, 'cause only bot users can place bans)
  set thereason [concat [join [list $hand ":"] ""] $reason]

  putserv "NOTICE $nick :Kickbanning $who ($userhost) from $channel ($reason)"
  putserv "MODE $channel -o+b $who $banmask"
  putserv "KICK $channel $who :$reason"
  addban $banmask $thereason
  return 1
}


#   ban <nick> [reason]
#    or:  ban <nick!user@host> [reason]

bind msg o ban msg_ban
proc msg_ban {nick uhost hand arg} {
  global channel botnick 
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick ban <nick> \[reason\] (if user is on channel)"
    putserv "NOTICE $nick :Usage: /msg $botnick ban <nick!user@host> \[reason\] (otherwise)"
    return 0
  }
  set who [lindex $arg 0]
  set reason [lrange $arg 1 end]
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }
  if {![botisop]} {
    putserv "NOTICE $nick :I'm not an op on the channel!  I can't help you."
    return 0
  }

  # Separate the "banmask" into the three parts: nick!user@host
  # If any of the parts is missing, don't allow the ban to be placed
  set nrparts [scan $who "%\[^!\]!%\[^@\]@%s" thenick theuser thehost]

  # For a nick ban, $nrparts == 1, for a full ban, $nrparts == 3
  if {$nrparts == 1} {
    if {![onchan $who]} {
      putserv "NOTICE $nick :You specified a nick ban, but the user is not on the channel!"
      return 0
    }
    append tempwho $who "!*" [getchanhost $who]
    set banmask [maskhost $tempwho]
  } elseif {$nrparts != 3} {
    putserv "NOTICE $nick :You must use a proper ban: nick!user@host.  Wildcards are allowed."
    return 0
  } else {
    set banmask $who
  }

  # Now let's see who they're trying to ban, and see if we know them
  set target [finduser $banmask]
  if {$target != "*"} {
    if {([matchattr $target f] || [matchattr $target m])} {
      putserv "NOTICE $nick :Can't ban a friend or master, sorry."
      return 0
    }
  }

  # If the ban is already on the channel, don't re-ban it
  if {[ischanban $banmask]} {
    putserv "NOTICE $nick :That ban is already on the channel."
    return 0
  }
  # If the ban is already on the bot, don't re-ban it
  if {[isban $banmask]} {
    putserv "NOTICE $nick :That ban is already on the bot.  It will be enforced if necessary."
    return 0
  }

  # Finally, tidy up the comment to include the banner's handle.
  # (handle cannot be null, 'cause only bot users can place bans)
  set thereason [concat [join [list $hand ":"] ""] $reason]

  putserv "NOTICE $nick :Banning $banmask from $channel ($reason)"
  putserv "MODE $channel +b $banmask"
  addban $banmask $thereason
  return 1
}


#   unban <nick!user@host>

bind msg o unban msg_unban
proc msg_unban {nick uhost hand arg} {
  global channel botnick 
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick unban <nick!user@host>"
    return 0
  }
  set banmask [lindex $arg 0]
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }
  if {![botisop]} {
    putserv "NOTICE $nick :I'm not an op on the channel!  I can't help you."
    return 0
  }
 
  # Separate the "banmask" into the three parts: nick!user@host
  # If any of the parts is missing, don't allow the ban to be placed
  set nrparts [scan $banmask "%\[^!\]!%\[^@\]@%s" thenick theuser thehost]

  if {$nrparts != 3} {
    putserv "NOTICE $nick :You must use a proper ban: nick!user@host.  Wildcards are allowed."
    return 0
  }

  # If the ban isn't on the channel, report it!
  if {(![ischanban $banmask] && ![isban $banmask])} {
    putserv "NOTICE $nick :That ban is not on the channel or the bot!"
    return 0
  }

  # If it's not a permban, remove it from the bot (eg automatically placed bans)
  # If it is a permban, just warn the user.
  # Note that the ban must be removed from the bot before the channel,
  # because if "enforce-bans" is true, the bot would only put the ban back!
  #if {![ispermban $banmask]} {
  #  delban $banmask
  #} else {
  #  putserv "NOTICE $nick :That's in my permban list.  You need to have a master use -BAN if you want it gone for good."
  #}

  putserv "NOTICE $nick :Unbanning $banmask from $channel"
  putserv "MODE $channel -b $banmask"

  return 1
}


#   giveop <nick> [nick ...]

bind msg o giveop msg_giveop
proc msg_giveop {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick giveop <nick> \[nick ...\]"
    return 0
  }
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }

  # We're gonna loop through the arguments, and op 'em 3 at a time.
  # Initialize the loop: oplist is the current group to be opped
  #                      opped includes all people successfully opped so far
  set oplist {}
  set opped {}

  foreach who $arg {
    if {![onchan $who]} {
      putserv "NOTICE $nick :$who is not on the channel!"
      continue
    }
    if {[onchanSplit $who]} {
      putserv "NOTICE $nick :$who is currently net-split."
      continue
    }
    # No point in opping myself!
    if {[string compare [string tolower $who] [string tolower $botnick]] == 0} {
      putserv "NOTICE $nick :Op myself?  Now you're just being plain silly!"
      continue
    }
    set userhost ""
    append userhost $who "!" [getchanhost $who]
    set target [finduser $userhost]
    if {$target != "*"} {
      if {[matchattr $target d]} {
        putserv "NOTICE $nick :Can't giveop to someone I'm deopping ($userhost)!"
        continue
      }
    }

    # OK, we'll op this person
    lappend oplist $who

    # Reached 3 yet?
    if {[llength $oplist] == 3} {
      putserv "MODE $channel +ooo [join $oplist]"
      lappend opped [join $oplist]
      set oplist {}
    }
  }

  # Loop is done -- now deal with any residuals 
  if {[llength $oplist] > 0} {
    putserv "MODE $channel +ooo [join $oplist]"
    lappend opped [join $oplist]
    set oplist {}
  }
  
  putserv "NOTICE $nick :Opped the following users on $channel: [join $opped]"
  return 1
}


#   deop   <nick> [nick ...]

bind msg o deop msg_deop
proc msg_deop {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick deop <nick> \[nick ...\]"
    return 0
  }
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }

  # We're gonna loop through the arguments, and op 'em 3 at a time.
  # Initialize the loop: deoplist is the current group to be opped
  #                      deopped includes all people successfully opped so far
  set deoplist {}
  set deopped {}

  foreach who $arg {
    if {![onchan $who]} {
      putserv "NOTICE $nick :$who is not on the channel!"
      continue
    }
    if {[onchanSplit $who]} {
      putserv "NOTICE $nick :$who is currently net-split."
      continue
    }
    # No point in deopping myself!
    if {[string compare [string tolower $who] [string tolower $botnick]] == 0} {
      putserv "NOTICE $nick :Deop myself?  Now you're just being plain silly!"
      continue
    }
    set userhost ""
    append userhost $who "!" [getchanhost $who]
    set target [finduser $userhost]
    if {$target != "*"} {
      if {([matchattr $target m] || [matchattr $target f])} {
        putserv "NOTICE $nick :Can't deop a master or friend ($userhost)"
        continue
      }
    }

    # OK, we'll deop this person
    lappend deoplist $who

    # Reached 3 yet?
    if {[llength $deoplist] == 3} {
      putserv "MODE $channel -ooo [join $deoplist]"
      lappend deopped [join $deoplist]
      set deoplist {}
    }
  }

  # Loop is done -- now deal with any residuals 
  if {[llength $deoplist] > 0} {
    putserv "MODE $channel -ooo [join $deoplist]"
    lappend deopped [join $deoplist]
    set deoplist {}
  }
  
  putserv "NOTICE $nick :Deopped the following users on $channel: [join $deopped]"
  return 1
}

