
# Eden's TCL Extensions To Eggdrop  (ETETE)
# by Wayne Brookes, February 1995
# version 0.1


# MASTER COMMANDS
#
# This file adds the following 'msg' commands:
#   +user <handle> <hostmask>
#   +host <handle> <hostmask>
#   -host <handle> <hostmask>
#   chattr <handle> [changes]
#   dump <server command>
#   set <varname> [value]
#   count
#   isuser <handle>
#   onchan <handle>
#   laston <handle>
#   +info <handle> <info>
#   -info <handle>
#   info? <handle>
#   +email <handle> <email>
#   -email <handle>
#   email? <handle>
#   +comment <handle> <comment>
#   -comment <handle>
#   comment? <handle>
#   +dccdir <handle> <dccdir>
#   -dccdir <handle>
#   dccdir? <handle>
#   +ban <nick!user@host> <reason>
#   -ban <nick!user@host>
#   ban? <nick!user@host>
#   +deop <handle>
#   +friend <handle>
#   +master <handle>
#   +op <handle>
#   +party <handle>
#   +xfer <handle>
#   +kick <handle>
#   +bot <handle> <host:port>
#   +auto <handle>
#   +reject <handle>
#   +leaf <handle>
#   +share <handle>


#   +user <handle> <hostmask>

bind msg m +user msg_+user
proc msg_+user {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +user <handle> <hostmask>"
    return 0
  }
  set handle [lindex $arg 0]
  set hostmask [lindex $arg 1]
  set target [finduser $hostmask]
  if {$target != "*"} {
    # the target is known by the bot!  
    putserv "NOTICE $nick :I just thought you should know that the new hostmask also matches the handle \"$target\""
  }

  set valid [validuser $handle]

  if {![adduser $handle $hostmask]} {
    putserv "NOTICE $nick :Couldn't add \"$handle\" to my user list."
    if {$valid} {
      putserv "NOTICE $nick :Somebody with that handle already exists!"
    } else {
      putserv "NOTICE $nick :I don't know what the problem was :-("
    }
    return 0
  }
  putserv "NOTICE $nick :Added \"$handle\" with the hostmask: $hostmask"
  return 1
}


#   +host <handle> <hostmask>

bind msg m +host msg_+host
proc msg_+host {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +host <handle> <hostmask>"
    return 0
  }
  set handle [lindex $arg 0]
  set hostmask [lindex $arg 1]

  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }

  # try and retreive any existing handle which matches the hostmask (if any)
  set target [finduser $hostmask]
  if {$target != "*"} {
    # the target is known by the bot!  
    if {$target == $handle} {
      putserv "NOTICE $nick :Umm, \"$handle\" already has that hostmask.  I won't add it again, if you don't mind."
      return 0
    } else {
      putserv "NOTICE $nick :I just thought you should know that the new hostmask also matches the handle \"$target\""
    }
  }

  addhost $handle $hostmask
  putserv "NOTICE $nick :Added \"$hostmask\" to \"$handle\""
  return 1
}


#   -host <handle> <hostmask>

bind msg m -host msg_-host
proc msg_-host {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick -host <handle> <hostmask>"
    return 0
  }
  set handle [lindex $arg 0]
  set hostmask [lindex $arg 1]

  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }

  if {![delhost $handle $hostmask]} {
    putserv "NOTICE $nick :Couldn't delete hostmask \"$hostmask\" from \"$handle\"!"
    return 0
  }

  putserv "NOTICE $nick :Deleted \"$hostmask\" from \"$handle\""
  return 1
}


#   chattr <handle> [changes]

bind msg m chattr msg_chattr
proc msg_chattr {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick chattr <handle> \[changes\]"
    putserv "NOTICE $nick :       e.g. the changes are \"+f\", \"-o\", \"+dk\", \"-o+d\""
    putserv "NOTICE $nick :       If you don't specify any changes, I will tell you what the user's current attributes are."
    return 0
  }
  set handle [lindex $arg 0]
  set changes [lindex $arg 1]

  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }

  set newflags [chattr $handle $changes]

  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   dump <server command>

bind msg m dump msg_dump
proc msg_dump {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick dump <server command>"
    return 0
  }

  putserv "$arg"
  putserv "NOTICE $nick :Dumped command to the server"

  return 1
}


#   set <varname> [value]

bind msg m set msg_set
proc msg_set {thisnick uhost hand arg} {
  global channel botnick botname server nick username realname home-channel
  global chanmode servers servlimit log-time clear-bans op-on-join revenge
  global greet learn-users require-x require-p enforce-bans periodic-save
  global use-info share-users passive strict-host isolate keep-all-logs
  global chan-updates perm-bans logfile console dcc-path dcc-incoming
  global dcc-quota telnet userfile notefile motd helpdir tempdir gain-ops
  global init-server admin helpbot flood-msg flood-chan flood-join ban-time
  global ignore-time notify-newusers default-flags channel-key raw-files
  global upload-to-cd topic

  if {$arg == ""} {
    putserv "NOTICE $thisnick :Usage: /msg $botnick set <varname> \[value\]"
    return 0
  }

  set varname [lindex $arg 0]
  set value "\"[join [lrange $arg 1 end]]\""

  if {$value == "\"\""} {
    set result [eval "set $varname"]
  } else {
    set result [eval "set $varname $value"]
  }

  putserv "NOTICE $thisnick :Value of \"$varname\" is: $result"

  return 1
}


#   count

bind msg m count msg_count
proc msg_count {nick uhost hand arg} {
  putserv "NOTICE $nick :My user list contains [countusers] users!"
  return 1
}


#   isuser <handle>

bind msg m isuser msg_isuser
proc msg_isuser {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick isuser <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {[validuser $handle]} {
    putserv "NOTICE $nick :Yep, I know someone by the handle of $handle!"
  } else {
    putserv "NOTICE $nick :Hmm, nope, I don't know anyone called $handle :-("
  }
  return 1
}


#   onchan <nick>

bind msg m onchan msg_onchan
proc msg_onchan {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick onchan <nick>"
    return 0
  }
  set handle [lindex $arg 0]
  if {[onchan $handle]} {
    putserv "NOTICE $nick :Yep, $handle is on the channel now!"
  } else {
    putserv "NOTICE $nick :Hmm, nope, I don't see $handle on the channel :-("
  }
  return 1
}


#   laston <handle>

bind msg m laston msg_laston
proc msg_laston {nick uhost hand arg} {
  global botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick laston <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }

  set thelaston [getlaston $handle]

  if {$thelaston == 0} {
    putserv "NOTICE $nick :$handle has never been seen on channel by me!"
  } else {
    putserv "NOTICE $nick :$handle was last seen on [ctime $thelaston]"
  }
  return 1
}


#   +info <handle> <info>

bind msg m +info msg_+info
proc msg_+info {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +info <handle> <description>"
    return 0
  }
  set handle [lindex $arg 0]
  set description [lrange $arg 1 end]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  setinfo $handle $description
  putserv "NOTICE $nick :New info for $handle is: $description"
  return 1
}

#   -info <handle>

bind msg m -info msg_-info
proc msg_-info {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick -info <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  setinfo $handle ""
  putserv "NOTICE $nick :Deleted info for $handle"
  return 1
}


#   info? <handle>

bind msg m info? msg_info?
proc msg_info? {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick info? <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set theinfo [getinfo $handle]
  if {$theinfo == ""} {
    putserv "NOTICE $nick :$handle does not have any info set!"
  } else {
    putserv "NOTICE $nick :Info for $handle is currently: $theinfo"
  }
  return 1
}


#   +email <handle> <email>

bind msg m +email msg_+email
proc msg_+email {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +email <handle> <email>"
    return 0
  }
  set handle [lindex $arg 0]
  set email [lrange $arg 1 end]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  setemail $handle $email
  putserv "NOTICE $nick :New email for $handle is: $email"
  return 1
}


#   -email <handle>

bind msg m -email msg_-email
proc msg_-email {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick -email <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  setemail $handle ""
  putserv "NOTICE $nick :Deleted email for $handle"
  return 1
}


#   email? <handle>

bind msg m email? msg_email?
proc msg_email? {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick email? <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set theemail [getemail $handle]
  if {$theemail == ""} {
    putserv "NOTICE $nick :$handle does not have any email address set!"
  } else {
    putserv "NOTICE $nick :Email for $handle is currently: $theemail"
  }
  return 1
}


#   +comment <handle> <comment>

bind msg m +comment msg_+comment
proc msg_+comment {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +comment <handle> <comment>"
    return 0
  }
  set handle [lindex $arg 0]
  set comment [lrange $arg 1 end]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  setcomment $handle $comment
  putserv "NOTICE $nick :New comment for $handle is: $comment"
  return 1
}


#   -comment <handle>  

bind msg m -comment msg_-comment
proc msg_-comment {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick -comment <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  setcomment $handle ""
  putserv "NOTICE $nick :Deleted comment for $handle"
  return 1
}


#   comment? <handle>

bind msg m comment? msg_comment?
proc msg_comment? {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick comment? <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set thecomment [getcomment $handle]
  if {$thecomment == ""} {
    putserv "NOTICE $nick :$handle does not have a comment!"
  } else {
    putserv "NOTICE $nick :Comment for $handle is currently: $thecomment"
  }
  return 1
}


#   +dccdir <handle> <dccdir>

bind msg m +dccdir msg_+dccdir
proc msg_+dccdir {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +dccdir <handle> <dccdir>"
    return 0
  }
  set handle [lindex $arg 0]
  set dccdir [lrange $arg 1 end]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  setdccdir $handle $dccdir
  putserv "NOTICE $nick :New dccdir for $handle is: $dccdir"
  return 1
}


#   -dccdir <handle>

bind msg m -dccdir msg_-dccdir
proc msg_-dccdir {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick -dccdir <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  setdccdir $handle ""
  putserv "NOTICE $nick :Deleted dccdir for $handle"
  return 1
}


#   dccdir? <handle>

bind msg m dccdir? msg_dccdir?
proc msg_dccdir? {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick dccdir? <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set thedccdir [getdccdir $handle]
  if {$thedccdir == ""} {
    putserv "NOTICE $nick :$handle does not have a dccdir!"
  } else {
    putserv "NOTICE $nick :DCCdir for $handle is currently: $thedccdir"
  }
  return 1
}


#   +ban <nick!user@host> <reason>

bind msg m +ban msg_+ban
proc msg_+ban {nick uhost hand arg} {
  global channel botnick 
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +ban <nick!user@host> <reason>"
    return 0
  }
  set banmask [lindex $arg 0]
  set reason [lrange $arg 1 end]
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!  I can place the ban on the bot, but not on the channel."
  }
  if {![botisop]} {
    putserv "NOTICE $nick :I'm not an op on the channel!  I can place the ban on the bot, but not on the channel."
  }
 
  # Separate the "banmask" into the three parts: nick!user@host
  # If any of the parts is missing, don't allow the ban to be placed
  set nrparts [scan $banmask "%\[^!\]!%\[^@\]@%s" thenick theuser thehost]

  if {$nrparts != 3} {
    putserv "NOTICE $nick :You must use a proper ban: nick!user@host.  Wildcards are allowed."
    return 0
  }

  # Now let's see who they're trying to ban, and see if we know them
  set target [finduser $banmask]

  if {$target != "*"} {
    if {([matchattr $target f] || [matchattr $target m])} {
      putserv "NOTICE $nick :Can't ban a friend or master, sorry."
      return 0
    }
  }

  # If the ban is already on the bot, warn the user
  if {[isban $banmask]} {
    putserv "NOTICE $nick :Note: that ban is already on the bot. Adding it as a permban anyway."
  }

  # For a permban, a reason is essential
  set firstreason [lindex $reason 0]
  if {$firstreason == ""} {
    putserv "NOTICE $nick :You \002MUST\002 specify a reason for a permban!"
    return 0
  }

  # Finally, tidy up the comment to include the banner's handle.
  # (handle cannot be null, 'cause only bot users can place bans)
  set thereason [concat [join [list $hand ":"] ""] $reason]

  putserv "NOTICE $nick :Adding $banmask as a permban ($reason)"
  putserv "NOTICE $nick :(I won't place the ban on channel right now though - if you want it to show up right away, \"/msg $botnick RESET\")"
  #putserv "MODE $channel +b $banmask"
  addpermban $banmask $thereason
  return 1
}


#   -ban <nick!user@host>

bind msg m -ban msg_-ban
proc msg_-ban {nick uhost hand arg} {
  global channel botnick 
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick -ban <nick!user@host>"
    return 0
  }
  set banmask [lindex $arg 0]
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!  I can remove the ban from the bot, but not the channel."
  }
  if {![botisop]} {
    putserv "NOTICE $nick :I'm not an op on the channel!  I can remove the ban from the bot, but not the channel."
  }
 
  # Separate the "banmask" into the three parts: nick!user@host
  # If any of the parts is missing, don't allow the ban to be placed
  set nrparts [scan $banmask "%\[^!\]!%\[^@\]@%s" thenick theuser thehost]

  if {$nrparts != 3} {
    putserv "NOTICE $nick :You must use a proper ban: nick!user@host.  Wildcards are allowed."
    return 0
  }

  # If the ban isn't on the bot's ban list, fail!
  if {![isban $banmask]} {
    putserv "NOTICE $nick :That's not in my ban list!  I'll see if I can find the ban on the channel (not placed by the bot)"
    if {[ischanban $banmask]} {
      putserv "NOTICE $nick :Yes, I've found it.  Removing the ban on $banmask from $channel"
      putserv "MODE $channel -b $banmask"
      return 1
    } else {
      putserv "NOTICE $nick :Nope, I can't find the ban $banmask anywhere.  Sorry!"
      return 0
    }
  }

  # If the ban isn't on the channel, warn the user.  Some bot bans aren't.
  if {![ischanban $banmask]} {
    putserv "NOTICE $nick :Note: That ban is not on the channel.  It doesn't matter, I'll still remove it from my internal ban list."
  }

  # If it is a permban, make sure the user knows!
  #if {[ispermban $banmask]} {
  #  putserv "NOTICE $nick :Note: You're about to remove a \002PERM\002 ban."
  #  set permban "\(permban\)"
  #} else { 
     set permban ""
  #}

  putserv "NOTICE $nick :Removing $banmask as a ban from the bot $permban"
  putserv "MODE $channel -b $banmask"
  delban $banmask
  return 1
}


#   ban? <nick!user@host>

bind msg m ban? msg_ban?
proc msg_ban? {nick uhost hand arg} {
  global channel botnick 
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick ban? <nick!user@host>"
    return 0
  }
  set banmask [lindex $arg 0]

  # Separate the "banmask" into the three parts: nick!user@host
  # If any of the parts is missing, don't allow the ban to be placed
  set nrparts [scan $banmask "%\[^!\]!%\[^@\]@%s" thenick theuser thehost]

  if {$nrparts != 3} {
    putserv "NOTICE $nick :You must use a proper ban: nick!user@host.  Wildcards are allowed."
    return 0
  }

  #if {[ispermban $banmask]} {
  #  putserv "NOTICE $nick :That's a permanent ban!"
  #} elseif {[isban $banmask]} {
  #  putserv "NOTICE $nick :That's a ban on my bot list (not permanent though)."
  #} else {
  #  putserv "NOTICE $nick :That's not a ban on the bot."
  #}
  if {[isban $banmask]} {
    putserv "NOTICE $nick :That's a ban on my bot list."
  } else {
    putserv "NOTICE $nick :That's not a ban on the bot."
  }
  if {[ischanban $banmask]} {
    putserv "NOTICE $nick :That ban is on the channel!"
  } else {
    putserv "NOTICE $nick :That ban isn't on the channel!"
  }
  return 1
}


#   +deop <handle>

bind msg m +deop msg_+deop
proc msg_+deop {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +deop <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set newflags [chattr $handle "+d"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +friend <handle>

bind msg m +friend msg_+friend
proc msg_+friend {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +friend <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set newflags [chattr $handle "+f"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +master <handle>

bind msg m +master msg_+master
proc msg_+master {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +master <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set newflags [chattr $handle "+m"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +op <handle>

bind msg m +op msg_+op
proc msg_+op {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +op <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set newflags [chattr $handle "+o"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +party <handle>

bind msg m +party msg_+party
proc msg_+party {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +party <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set newflags [chattr $handle "+p"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +xfer <handle>

bind msg m +xfer msg_+xfer
proc msg_+xfer {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +xfer <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set newflags [chattr $handle "+x"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +kick <handle>

bind msg m +kick msg_+kick
proc msg_+kick {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +kick <handle>"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any user with the handle \"$handle\"!"
    return 0
  }
  set newflags [chattr $handle "+k"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#  +bot <handle> <host:port>

bind msg m +bot msg_+bot
proc msg_+bot {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +bot <handle> <address>
    putserv "NOTICE $nick :       The <address> is of the form \"host:port\""
    return 0
  }
  set handle [lindex $arg 0]
  set address [lindex $arg 1]
  if {[validuser $handle]} {
    putserv "NOTICE $nick :That handle is already in use!"
    return 0
  }

  # break the address up into host and port sections
  set nrparts [scan $address "%\[^:\]:%s" thehost theport]
  if {$nrparts != 2} {
    putserv "NOTICE $nick :You must specify a proper address of the form \"host:port\""
    return 0
  }
  if {![adduser $handle "none"]} {
    putserv "NOTICE $nick :I had a problem creating a userlist entry for the new bot."
    return 0
  }
  chattr $handle "+t-odbkmfxipsalr"
  setinfo $handle $address
  putserv "NOTICE $nick :Added a new tandem bot entry for $handle at $address"
  return 1
}


#   +auto <handle>

bind msg m +auto msg_+auto
proc msg_+auto {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +auto <handle>"
    putserv "NOTICE $nick :       Remember, +auto is only for tandem bots!"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any tandem bots with the handle \"$handle\"!"
    return 0
  }
  if {![matchattr $handle "t"]} {
    putserv "NOTICE $nick :But $handle is not a tandem bot!"
    return 0
  }
  set newflags [chattr $handle "+a"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +reject <handle>

bind msg m +reject msg_+reject
proc msg_+reject {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +reject <handle>"
    putserv "NOTICE $nick :       Remember, +reject is only for tandem bots!"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any tandem bots with the handle \"$handle\"!"
    return 0
  }
  if {![matchattr $handle "t"]} {
    putserv "NOTICE $nick :But $handle is not a tandem bot!"
    return 0
  }
  set newflags [chattr $handle "+r"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +leaf <handle>

bind msg m +leaf msg_+leaf
proc msg_+leaf {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +leaf <handle>"
    putserv "NOTICE $nick :       Remember, +leaf is only for tandem bots!"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any tandem bots with the handle \"$handle\"!"
    return 0
  }
  if {![matchattr $handle "t"]} {
    putserv "NOTICE $nick :But $handle is not a tandem bot!"
    return 0
  }
  set newflags [chattr $handle "+l"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


#   +share <handle>

bind msg m +share msg_+share
proc msg_+share {nick uhost hand arg} {
  global channel botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick +share <handle>"
    putserv "NOTICE $nick :       Remember, +share is only for tandem bots!"
    return 0
  }
  set handle [lindex $arg 0]
  if {![validuser $handle]} {
    putserv "NOTICE $nick :But I don't know any tandem bots with the handle \"$handle\"!"
    return 0
  }
  if {![matchattr $handle "t"]} {
    putserv "NOTICE $nick :But $handle is not a tandem bot!"
    return 0
  }
  set newflags [chattr $handle "+s"]
  putserv "NOTICE $nick :The flags for $handle are currently \"$newflags\"."
  return 1
}


