
# Eden's TCL Extensions To Eggdrop  (ETETE)
# by Wayne Brookes, February 1995
# version 0.1


# MISCELLANOUS COMMANDS
#
# This file adds the following 'msg' commands:
#   names
#   log <text>
#   time
#   showinfo [nick]
#   invite [nick]
#   say <text>
#   me <action>
#     or act <action>
#   msg <nick> <msg>
#   do <nick> <action>
#   topic [topic]


#   names

bind msg - names msg_names
proc msg_names {nick uhost hand arg} {
  global channel
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }

  set users [chanlist]

  # Let's add a "@" in front of the chanops, just like IrcII does.
  foreach user $users {
    if {[isop $user]} { 
      append userlist "@" $user " " 
    } else {
      append userlist $user " "
    }
  }

  append userhost $nick "!" $uhost
  set target [finduser $userhost]
  if {($target != "*" && $target != "*ban")} {
    putserv "NOTICE $nick :Users on $channel: $userlist"
  } else {
    putserv "NOTICE $nick :Sorry, I don't think I know you"
    return 0
  }
  return 1
}


#   log <text>

bind msg - log msg_log
proc msg_log {nick uhost hand arg} {
  global botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick log <text to add to log>"
    return 0
  }

  putlog [concat "USERLOG \($nick!$uhost]\):" $arg]
  putserv "NOTICE $nick :Added text to the bot log as requested."
  return 1
}


#   time

bind msg - time msg_time
proc msg_time {nick uhost hand arg} {
  global botnick
  set curtime [ctime [unixtime]]
  putserv "NOTICE $nick :Current time at $botnick's home: $curtime"
  return 1
}


#   showinfo [nick]

bind msg - showinfo msg_showinfo
proc msg_showinfo {nick uhost hand arg} {
  global channel
  set who [lindex $arg 0]
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }
  append userhost $nick "!" $uhost
  set handle [finduser $userhost]
  if {$handle != "*"} {

    # Two main branches here: one if the user is a friend or master,
    # and the other if the user isn't.  Friends and masters are allowed to
    # display other people's info ... other users can only display their own.

    if {([matchattr $handle f] || [matchattr $handle m])} {
      # By default, display user's own info if no-one else specified
      if {$who == ""} { set who $handle }
      # Do I know them?
      if {![validuser $who]} {
	putserv "NOTICE $nick :But I don't know a user called $who"
	return 0
      } else {
	set theinfo [getinfo $who]
	# Do they have any info set?
	if {$theinfo == ""} {
	  putserv "NOTICE $nick :But $who doesn't have any info set!"
	  return 0
        } else {
	  putchan "\[$who\] $theinfo"
	  return 1
        } 
      }
    } else {
      if {($who == "" || $who == $nick || $who == $handle)} {
	set theinfo [getinfo $handle]
	if {$theinfo == ""} {
	  putserv "NOTICE $nick :But you don't have any info set!"
	  return 0
        } else {
	  putchan "\[$nick\] $theinfo"
	  return 1
        } 
      } else {
        putserv "NOTICE $nick :Sorry, you have to be a friend or master to display other people's info"
        return 0
      }
    }
  }

# just in case ... should never reach here though!
return 0
}


#   invite [nick]

# Overrides the default eggdrop invite command.  This version will invite
# you to the channel if you don't specify any nick, or will invite another
# user to join the channel if you do.
bind msg - invite msg_invite
proc msg_invite {nick uhost hand arg} {
  global channel 
  if {$channel == {}} {
    putserv "NOTICE $nick :I'm not on a channel!"
    return 0
  }

  # No user specified?  Default to inviting the invoker.
  set who [lindex $arg 0]
  if {$who == ""} {set who "$nick"}

  if {[onchan $who]} {
    if {"$who" == "$nick"} {
      putserv "NOTICE $nick :You are already on $channel!"
    } else {
      putserv "NOTICE $nick :$who is already on $channel!"
    }
    return 0
  }

  append userhost $nick "!" $uhost
  set target [finduser $userhost]
  if {($target != "*" && $target != "*ban")} {
    if {"$who" != "$nick"} {
      putserv "NOTICE $nick :Inviting $who to $channel"
    }
    putserv "INVITE $who :$channel" 
  } else {
    putserv "NOTICE $nick :Sorry, I don't think I know you"
    return 0
  }
  return 1
}


#   say <text>

bind msg - say msg_say
proc msg_say {nick uhost hand arg} {
  global botnick channel
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick say <text to say to the channel>"
    return 0
  }
  #Find out if I know the user or not (and that they're not a ban)
  append userhost $nick "!" $uhost
  set target [finduser $userhost]
  if {($target == "*" || $target == "*ban")} {
    putserv "NOTICE $nick :Sorry, I don't think I know you"
    return 0
  } 

  putserv "PRIVMSG $channel :$arg"
  return 1
}


#   me <action>
#     or act <action>

bind msg - me msg_me
bind msg - act msg_me
proc msg_me {nick uhost hand arg} {
  global botnick channel
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick me <action to perform on channel>"
    return 0
  }
  #Find out if I know the user or not (and that they're not a ban)
  append userhost $nick "!" $uhost
  set target [finduser $userhost]
  if {($target == "*" || $target == "*ban")} {
    putserv "NOTICE $nick :Sorry, I don't think I know you"
    return 0
  }

  putserv "PRIVMSG $channel :\001ACTION $arg\001"
  return 1
}


#   msg <nick> <msg>

bind msg - msg msg_msg
proc msg_msg {nick uhost hand arg} {
  global botnick
  if {$arg == ""} {
    putnote $nick "Usage: /msg $botnick msg <nick> <msg>"
    return 0
  }
  set who [lindex $arg 0]
  set themsg [lrange $arg 1 end]
  #Find out if I know the user or not (and that they're not a ban)
  append userhost $nick "!" $uhost
  set target [finduser $userhost]
  if {($target == "*" || $target == "*ban")} {
    putserv "NOTICE $nick :Sorry, I don't think I know you"
    return 0
  } elseif {(($target != "*") && (![matchattr $target f] && ![matchattr $target m]))} {
    # OK, I do know them, but they're not a friend or master
    putserv "NOTICE $nick :You must be a friend or master to use MSG"
    return 0
  }

  putserv "PRIVMSG $who :$themsg"
  return 1
}


#   do <nick> <action>

bind msg - do msg_do
proc msg_do {nick uhost hand arg} {
  global botnick
  if {$arg == ""} {
    putserv "NOTICE $nick :Usage: /msg $botnick do <nick> <action>"
    return 0
  }
  set who [lindex $arg 0]
  set themsg [lrange $arg 1 end]
  #Find out if I know the user or not (and that they're not a ban)
  append userhost $nick "!" $uhost
  set target [finduser $userhost]
  if {($target == "*" || $target == "*ban")} {
    putserv "NOTICE $nick :Sorry, I don't think I know you"
    return 0
  } elseif {(($target != "*") && (![matchattr $target f] && ![matchattr $target m]))} {
    # OK, I do know them, but they're not a friend or master
    putserv "NOTICE $nick :You must be a friend or master to use DO"
    return 0
  }

  putserv "PRIVMSG $who :\001ACTION $themsg\001"
  return 1
}


#   topic [topic]

bind msg - topic msg_topic
proc msg_topic {nick uhost hand arg} {
  global channel topic

  # Assumes the "default" channel topic is contained in a variable called
  # "topic" (best defined in the bot's config file)

  # If no topic specified as an argument, reset the default, otherwise set
  # the topic given.
  # FUTURE: only ops allowed to set topic, but any user can reset the default
  if {[lindex $arg 0] == ""} {
    putserv "TOPIC $channel :[join $topic]"
  } else { 
    putserv "TOPIC $channel :[join $arg]"
  }

  return 1
}

